#include "nivelEth.h"
#include <pthread.h>
#include "rc_funcs.h"



/******************************
*Arquitectura de Redes II     *
*Funciones de Nivel Ethernet  *
*Javier Ramos 2013-2014       *
* Manuel Ruiz 2017-2018       *
*******************************/


pthread_t t;
pcap_t  *p;



/************************************************************************************************
*Funcion:getMACAddr 									                                     	*
*Descripcion: Obtiene la direccion MAC de una interfaz						                    *
*Entrada:											                                            * 
*	mac:vector de 6 bytes a rellenar con la MAC de la interfaz				                    *
*	interface: nombre la de interfaz de la cual se obtendra la MAC. Por ejemplo "eth0"	        *
*Retorno: Ninguno										                                        *
*************************************************************************************************/

void getMACAddr(uint8_t *mac,char *interface)
{
    
    int s;
    struct ifreq ifr;
    s = socket(AF_INET, SOCK_DGRAM, 0);
    strcpy(ifr.ifr_name, interface);
    ioctl(s, SIOCGIFHWADDR, &ifr);
    memcpy(mac,ifr.ifr_hwaddr.sa_data,HWADDR_len);
  
}
/************************************************************************************************
*Funcion:attendPacket										*
*Descripcion: Esta funcion se ejecuta cada vez que se recibe un paquete. Aqui dentro debe	*
*	      ejecutarse cualquier operación sobre el paquete recibido				*
*Entrada:											* 
*	user:informacion pasada a nivel de usuario por la libreria pcap (desde pcap_loop)	*
*	h: esructura de cabecera pcap del paquete recibido (tiempo y tamanyo)			*
*	packet: buffer con el contenido binario del paquete					*
*Retorno: Ninguno										*
*************************************************************************************************/

void attendPacket(u_char *user,const struct pcap_pkthdr *h,const u_char *packet)
{
	/**PRACTICA Implementar aquí las acciones necesarias al recibir un paquete*/
		printf(" IMPLEMENTAR attendPacket \n");
}


/************************************************************************************************
*Funcion:startCapture										*
*Descripcion: Esta funcion se ejecuta en un hilo nuevo e inicia la captura de paquetes a	*
*	      través de la función pcap_loop. Se ejecuta en un nuevo hilo parta evitar que el 	*
*	      programa principal quede bloqueado al llamar a la función InicializarNivel1a	*
*Entrada:											* 
*	arg: argumentos del hilo. En nuestro caso ninguno					*
*Retorno: Ninguno										*
*************************************************************************************************/
void * startCapture(void *arg)
{
	pcap_loop(p,-1,attendPacket,NULL);
}
/************************************************************************************************
*Funcion:InicializarStackRed							*
*Descripcion: Esta funcion inicia el nivel1a registrando la función de notificación, abriendo 	*
*	      una captura live pcap y lanzando un hilo de proceso de paquetes			*
*												*
*************************************************************************************************/
int InicializarEth(uint16_t *Tipos, uint8_t nType, tpfNotificacionRecepcionEth funcion, int timeout)
{
	char errbuff[1000];
	/**PRACTICA Implementar aquí las acciones necesarias para inciar el nivel 1a*/
	
	
	p = pcap_open_live("eth0",ETH_MAX_LEN,1,timeout,errbuff); //abre la interfaz eth0 en modo promiscuo, es decir, analiza todas las tramas que le llegan, tenemos que ver si son para nosotros o no (ver la direccion de destino) 
	if (p == NULL){
		printf ("Error: %s\n", errbuff);//el error se guarda en errbuff
		return ETH_ERROR;
	}
	pthread_create(&t,NULL,startCapture,NULL);
	printf(" IMPLEMENTAR InicializarEth\n");
	return 0;
	
}

/************************************************************************************************
*Funcion:FinalizarEth							*
*Descripcion: Esta funcion finaliza el nivel1a liberando los recursos que se hayan reservado y 	*
*	      cerrando la captura live pcap							*
*												*
*************************************************************************************************/
int FinalizarEth(void)
{
	/**PRACTICA Implementar aquí las acciones necesarias para finalizar el nivel 1a*/
		printf(" IMPLEMENTAR FinalizarStackRed \n");
	return 0;
}

/************************************************************************************************
*Funcion:EnviarDatagramaEth									*
*Descripcion: Esta funcion anyade una cabecera Ethernet y el CRC al mensaje pasado 		*
*             como argumento y lo envía utilizando la funcion pcap_inject			*
*												*
*************************************************************************************************/

int EnviarDatagramaEth(const uint8_t *direccion_destino, const  uint8_t *mensaje, uint16_t tamano, uint16_t tipo)
{
/**PRACTICA Implementar aquí las acciones necesarias para enviar un paquete*/
	printf(" IMPLEMENTAR EnviarDatagramaEth \n");
	
	
	return 0;
	

}
