#include "rc_funcs.h"

unsigned reverse(unsigned x) {
	x = ((x & 0x55555555) <<  1) | ((x >>  1) & 0x55555555);
	x = ((x & 0x33333333) <<  2) | ((x >>  2) & 0x33333333);
	x = ((x & 0x0F0F0F0F) <<  4) | ((x >>  4) & 0x0F0F0F0F);
	x = (x << 24) | ((x & 0xFF00) << 8) |
		((x >> 8) & 0xFF00) | (x >> 24);
	return x;
}

/********************************************************************************
*Funcion: crc32								*
*Descripcion: Esta funcion calcula el CRC de un conjunto de datos		*

*Entrada:									*
*	data: Array de datos sobre los que calcular el CRC			*
*	len: longitud de los datos sobre los que calcular el CRC		*
*Salida: Esta funcion devuelve el valor del CRC calculado (4 bytes)		*
*										*
*********************************************************************************/
uint32_t crc32(uint8_t *data,  uint16_t len) { 
	int i, j;
	unsigned int byte, crc;

	i = 0;
	crc = 0xFFFFFFFF;
	for (i = 0; i < len; i++) {
		byte = data[i];            // Get next byte.
		byte = reverse(byte);         // 32-bit reversal.
		for (j = 0; j <= 7; j++) {    // Do eight times.
			if ((int)(crc ^ byte) < 0)
				crc = (crc << 1) ^ 0x04C11DB7;
			else crc = crc << 1;
			byte = byte << 1;          // Ready next msg bit.
		}
	}
	return reverse(~crc);
}
/********************************************************************************
*Funcion: checksum								*
*Descripcion: Esta funcion calcula el checksum de un conjunto de datos		*

*Entrada:									*
*	data: Array de datos sobre los que calcular el checksum			*
*	len: longitud de los datos sobre los que calcular el checksum		*
*Salida: Esta funcion devuelve el valor del checksum calculado (1 bytes)		*
*										*
*********************************************************************************/

uint8_t checksum (uint8_t *data, uint16_t len) {
    unsigned char chk = 0;
    while (len-- != 0)
        chk -= *data++;
    return chk;
}


#define CRC16 0x8005
/* CRC-16	0x8005	x16 + x15 + x2 + 1*/

uint16_t crc16( uint8_t *data, uint16_t size)
{
    uint16_t out = 0;
    int bits_read = 0, bit_flag;

    /* Sanity check: */
    if(data == NULL)
        return 0;

    while(size > 0)
    {
        bit_flag = out >> 15;

        /* Get next bit: */
        out <<= 1;
        out |= (*data >> (7 - bits_read)) & 1;

        /* Increment bit counter: */
        bits_read++;
        if(bits_read > 7)
        {
            bits_read = 0;
            data++;
            size--;
        }

        /* Cycle check: */
        if(bit_flag)
            out ^= CRC16;
    }

    return out;
}

